// -----------------------------------------------------------------------------
//
// Copyright 2012(c) Analog Devices, Inc.
//
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without modification,
// are permitted provided that the following conditions are met:
//  - Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//  - Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in
//    the documentation and/or other materials provided with the
//    distribution.
//  - Neither the name of Analog Devices, Inc. nor the names of its
//    contributors may be used to endorse or promote products derived
//    from this software without specific prior written permission.
//  - The use of this software may or may not infringe the patent rights
//    of one or more patent holders.  This license does not release you
//    from the requirement that you obtain separate licenses from these
//    patent holders to use this software.
//  - Use of the software either in source or binary form, must be run
//    on or directly connected to an Analog Devices Inc. component.
//
// THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED
// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
// IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// -----------------------------------------------------------------------------
// FILE NAME : AD7693_vlg_tst.v
// MODULE NAME : AD7693_vlg_tst
// AUTHOR : acostina
// AUTHORS EMAIL : adrian.costina@analog.com
// -----------------------------------------------------------------------------
// SVN REVISION: 1464
// -----------------------------------------------------------------------------
// KEYWORDS : AD7693, testbench
// -----------------------------------------------------------------------------
// PURPOSE : Testbench for AD7693 driver
// -----------------------------------------------------------------------------
// REUSE ISSUES
// Reset Strategy      : Active low reset signal
// Clock Domains       : 100 MHz system clock
//                       50 MHz ADC sclk
// Critical Timing     : N/A
// Test Features       : N/A
// Asynchronous I/F    : N/A
// Instantiations      : N/A
// Synthesizable (y/n) : N 
// Target Device       : AD7693
// Other               : The driver is intended to be used for AD7693 ADCs configured
//                     : in /CS MODE, 3-WIRE, WITHOUT BUSY INDICATOR
// -----------------------------------------------------------------------------
// -----------------------------------------------------------------------------

`timescale 1 ns/ 100 ps

module AD7693_vlg_tst();
// test vector input registers
reg [15:0]  adc_generated_data;
reg         adc_clk_i;
reg         adc_sdi;
reg         adc_sdo;
reg         fpga_clk_i;
reg         reset_n_i;
// wires
wire        adc_cnv_o;
wire        adc_sclk_o;
wire [15:0] data_o;
wire        data_rd_ready_o;
reg [15:0]  adc_data;

// assign statements (if any)
AD7693 i1 (
// port map - connection between master ports and signals/registers
    .adc_clk_i(adc_clk_i),
    .adc_cnv_o(adc_cnv_o),
    .adc_sclk_o(adc_sclk_o),
    .adc_sdi(adc_sdi),
    .adc_sdo(adc_sdo),
    .data_o(data_o),
    .data_rd_ready_o(data_rd_ready_o),
    .fpga_clk_i(fpga_clk_i),
    .reset_n_i(reset_n_i)
);
initial
begin
    fpga_clk_i = 1'b0;
    adc_clk_i  = 1'b0;
    reset_n_i  = 1'b0;
    adc_data   = 16'h8001;
    adc_generated_data = 16'h8001;
#10
    reset_n_i = 1'b1;
$display("Running testbench");
end

//main clock generation
always
begin
    #5 fpga_clk_i <= ~fpga_clk_i;
end

//adc clock generation
always
begin
    #10 adc_clk_i <= ~adc_clk_i;
end

//simulate ADC behaviour
always @(negedge adc_sclk_o, posedge data_rd_ready_o)
begin
    if(data_rd_ready_o == 1'b1)
    begin
        adc_generated_data  <= adc_generated_data + 1;
        adc_data            <= {adc_generated_data[14:0],1'b0};
        #18 adc_sdo         <= adc_generated_data[15]; //simulates Ten
    end
    else
    begin
        #17 adc_sdo  <= adc_data[15];
        adc_data <= {adc_data[14:0],adc_data[15]};
    end
end

endmodule

